﻿namespace TabsStudioSync
{
    public class Sync : TabsStudioExt.ITabsStudioAddin
    {
        public Sync()
        {
            System.Windows.Controls.Image tabImage = new System.Windows.Controls.Image();
            tabImage.Source = new System.Windows.Media.Imaging.BitmapImage(
                new System.Uri("pack://application:,,,/Sync;component/sychronize.png"));

            System.Windows.Controls.Image tabExtensionImage = new System.Windows.Controls.Image();
            tabExtensionImage.Source = new System.Windows.Media.Imaging.BitmapImage(
                new System.Uri("pack://application:,,,/Sync;component/sychronize.png"));

            syncTabExtensionMenuItem = new System.Windows.Controls.MenuItem();
            syncTabExtensionMenuItem.Icon = tabExtensionImage;
            syncTabExtensionMenuItem.Header = "Sync with Solution Explorer";
            syncTabExtensionMenuItem.Click += SyncTabExtension;

            syncTabMenuItem = new System.Windows.Controls.MenuItem();
            syncTabMenuItem.Icon = tabImage;
            syncTabMenuItem.Header = "Sync group with Solution Explorer";
            syncTabMenuItem.Click += SyncTab;
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;
            engine.OpeningTabExtensionContextMenu += OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu += OpeningTabContextMenu;
        }

        public void OnDisconnection()
        {
            engine.OpeningTabExtensionContextMenu -= OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu -= OpeningTabContextMenu;
        }

        private void OpeningTabExtensionContextMenu(object sender, 
            TabsStudioExt.OpeningTabExtensionContextMenuEventArgs e)
        {
            try
            {
                if (e.TabExtension.Window.Document == null || e.TabExtension.Window.Document.ProjectItem == null)
                    return;
            }
            catch (System.Exception)
            {
                return;
            }
            
            storedTabExtension = e.TabExtension;
            PlaceMenuItem(syncTabExtensionMenuItem, e.ContextMenu);
        }

        private void OpeningTabContextMenu(object sender, TabsStudioExt.OpeningTabContextMenuEventArgs e)
        {
            storedTab = e.Tab;
            PlaceMenuItem(syncTabMenuItem, e.ContextMenu);
        }

        private void SyncTabExtension(object sender, System.Windows.RoutedEventArgs e)
        {
            try
            {
                EnvDTE.UIHierarchy solutionExplorer = dte.ToolWindows.SolutionExplorer;
                solutionExplorer.Parent.Activate();
                EnvDTE.ProjectItem projectItem = storedTabExtension.Window.Document.ProjectItem;
                projectItem.ExpandView();

                EnvDTE.UIHierarchyItem hierarchyItem = 
                    FindHierarchyItem(solutionExplorer.UIHierarchyItems, projectItem);
                if (hierarchyItem != null)
                    hierarchyItem.Select(EnvDTE.vsUISelectionType.vsUISelectionTypeSelect);
            }
            catch (System.Exception)
            {
            }
        }

        private void SyncTab(object sender, System.Windows.RoutedEventArgs e)
        {
            try
            {
                EnvDTE.UIHierarchy solutionExplorer = dte.ToolWindows.SolutionExplorer;
                solutionExplorer.Parent.Activate();

                bool firstItem = true;
                foreach (TabsStudioExt.TabExtension tabExtension in storedTab.TabExtensions)
                {
                    EnvDTE.ProjectItem projectItem = tabExtension.Window.Document.ProjectItem;
                    if (projectItem == null)
                        continue;
                    projectItem.ExpandView();

                    EnvDTE.UIHierarchyItem hierarchyItem =
                        FindHierarchyItem(solutionExplorer.UIHierarchyItems, projectItem);
                    if (hierarchyItem != null)
                    {
                        if (firstItem)
                        {
                            hierarchyItem.Select(EnvDTE.vsUISelectionType.vsUISelectionTypeSelect);
                            firstItem = false;
                        }
                        else
                        {
                            if (!hierarchyItem.IsSelected)
                                hierarchyItem.Select(EnvDTE.vsUISelectionType.vsUISelectionTypeToggle);
                        }
                    }
                }
            }
            catch (System.Exception)
            {
            }
        }

        private EnvDTE.UIHierarchyItem FindHierarchyItem(EnvDTE.UIHierarchyItems items, EnvDTE.ProjectItem p)
        {
            if (items == null)
                return null;

            foreach (EnvDTE.UIHierarchyItem item in items)
            {
                if (item.Object == p)
                    return item;
                EnvDTE.UIHierarchyItem result = FindHierarchyItem(item.UIHierarchyItems, p);
                if (result != null)
                    return result;
            }
            return null;
        }

        private void PlaceMenuItem(System.Windows.Controls.MenuItem menuItem,
            System.Windows.Controls.ContextMenu menu)
        {
            foreach (object item in menu.Items)
            {
                if (item is System.Windows.Controls.Control)
                {
                    if ((item as System.Windows.Controls.Control).Name.Equals("OpenContainingFolder"))
                    {
                        menu.Items.Insert(menu.Items.IndexOf(item) + 1, menuItem);
                        break;
                    }
                }
            }
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private TabsStudioExt.TabExtension storedTabExtension;
        private System.Windows.Controls.MenuItem syncTabExtensionMenuItem;
        private TabsStudioExt.Tab storedTab;
        private System.Windows.Controls.MenuItem syncTabMenuItem;
    }
}